/**
*     Copyright (c) 2023, Nsing Technologies Plt. Ltd
*
*     All rights reserved.
*
*     This software is the exclusive property of Nsingechnologies Plt. Ltd (Hereinafter
* referred to as NSING). This software, and the product of NSING described herein
* (Hereinafter referred to as the Product) are owned by NSING under the laws and treaties
* of the People's Republic of China and other applicable jurisdictions worldwide.
*
*     NSING does not grant any license under its patents, copyrights, trademarks, or other
* intellectual property rights. Names and brands of third party may be mentioned or referred
* thereto (if any) for identification purposes only.
*
*     NSING reserves the right to make changes, corrections, enhancements, modifications, and
* improvements to this software at any time without notice. Please contact NSING and obtain
* the latest version of this software before placing orders.

*     Although NSING has attempted to provide accurate and reliable information, NSING assumes
* no responsibility for the accuracy and reliability of this software.
*
*     It is the responsibility of the user of this software to properly design, program, and test
* the functionality and safety of any application made of this information and any resulting product.
* In no event shall NSING be liable for any direct, indirect, incidental, special,exemplary, or
* consequential damages arising in any way out of the use of this software or the Product.
*
*     NSING Products are neither intended nor warranted for usage in systems or equipment, any
* malfunction or failure of which may cause loss of human life, bodily injury or severe property
* damage. Such applications are deemed, "Insecure Usage".
*
*     All Insecure Usage shall be made at user's risk. User shall indemnify NSING and hold NSING
* harmless from and against all claims, costs, damages, and other liabilities, arising from or related
* to any customer's Insecure Usage.

*     Any express or implied warranty with regard to this software or the Product, including,but not
* limited to, the warranties of merchantability, fitness for a particular purpose and non-infringement
* are disclaimed to the fullest extent permitted by law.

*     Unless otherwise explicitly permitted by NSING, anyone may not duplicate, modify, transcribe
* or otherwise distribute this software for any purposes, in whole or in part.
*
*     NSING products and technologies shall not be used for or incorporated into any products or systems
* whose manufacture, use, or sale is prohibited under any applicable domestic or foreign laws or regulations.
* User shall comply with any applicable export control laws and regulations promulgated and administered by
* the governments of any countries asserting jurisdiction over the parties or transactions.
**/

/**
 *\*\file usbd_msc_cdc_composite.c
 *\*\author Nsing
 *\*\version v1.0.0
 *\*\copyright Copyright (c) 2023, Nsingechnologies Plt. Ltd All rights reserved.
 **/
#include "usbd_msc_core.h"
#include "usbd_cdc_core.h"
#include "usbd_msc_cdc_composite.h"
#include "usbd_desc.h"
#include "usbd_req.h"

/* CDC Device library callbacks */
extern uint8_t USBD_CDC_Init(void *pdev, uint8_t cfgidx);
extern uint8_t USBD_CDC_DeInit(void *pdev, uint8_t cfgidx);
extern uint8_t USBD_CDC_Setup(void *pdev, USB_SETUP_REQ *req);
extern uint8_t USBD_CDC_EP0_RxReady(void *pdev);
extern uint8_t USBD_CDC_DataIn(void *pdev, uint8_t epnum);
extern uint8_t USBD_CDC_DataOut(void *pdev, uint8_t epnum);
extern uint8_t USBD_CDC_SOF(void *pdev);

/* MSC Device library callbacks */
extern uint8_t USBD_MSC_Init(void *pdev, uint8_t cfgidx);
extern uint8_t USBD_MSC_DeInit(void *pdev, uint8_t cfgidx);
extern uint8_t USBD_MSC_Setup(void *pdev, USB_SETUP_REQ *req);
extern uint8_t USBD_MSC_DataIn(void *pdev, uint8_t epnum);
extern uint8_t USBD_MSC_DataOut(void *pdev, uint8_t epnum);

static uint8_t USBD_MSC_CDC_Init(void *pdev, uint8_t cfgidx);
static uint8_t USBD_MSC_CDC_DeInit(void *pdev, uint8_t cfgidx);

/* Control Endpoints*/
static uint8_t USBD_MSC_CDC_Setup(void *pdev, USB_SETUP_REQ *req);
static uint8_t USBD_MSC_CDC_EP0_RxReady(void *pdev);

/* Class Specific Endpoints*/
static uint8_t USBD_MSC_CDC_DataIn(void *pdev, uint8_t epnum);
static uint8_t USBD_MSC_CDC_DataOut(void *pdev, uint8_t epnum);
static uint8_t USBD_MSC_CDC_SOF(void *pdev);
static uint8_t *USBD_MSC_CDC_GetConfigDescriptor(uint8_t speed, uint16_t *length);

#define USB_MSC_CDC_CONFIG_DESC_SIZ (USB_MSC_CONFIG_DESC_SIZ - 9 + USB_CDC_CONFIG_DESC_SIZ + 8)

#define MSC_INTERFACE 0x00
#define CDC_COM_INTERFACE 0x01

USBD_Class_cb_TypeDef USBD_MSC_CDC_cb =
    {
        USBD_MSC_CDC_Init,
        USBD_MSC_CDC_DeInit,
        USBD_MSC_CDC_Setup,
        NULL,
        USBD_MSC_CDC_EP0_RxReady,
        USBD_MSC_CDC_DataIn,
        USBD_MSC_CDC_DataOut,
        USBD_MSC_CDC_SOF,
        NULL,
        NULL,
        USBD_MSC_CDC_GetConfigDescriptor,
        USBD_MSC_CDC_GetConfigDescriptor,
};

#ifdef USB_INTERNAL_DMA_ENABLED
#if defined(__ICCARM__) /*!< IAR Compiler */
#pragma data_alignment = 4
#endif
#endif /* USB_INTERNAL_DMA_ENABLED */
/* USB MSC/CDC device Configuration Descriptor */
__ALIGN_BEGIN static uint8_t USBD_MSC_CDC_CfgDesc[USB_MSC_CDC_CONFIG_DESC_SIZ] __ALIGN_END =
    {
        0x09,                              /* bLength: Configuration Descriptor size */
        USB_CONFIGURATION_DESCRIPTOR_TYPE, /* bDescriptorType: Configuration */
        USB_MSC_CDC_CONFIG_DESC_SIZ,
        /* wTotalLength: Bytes returned */
        0x00,
        0x03, /*bNumInterfaces: 3 interfaces (2 for CDC, 1 for MSC)*/
        0x01, /*bConfigurationValue: Configuration value*/
        0x00, /*iConfiguration: Index of string descriptor describing
the configuration*/
        0xC0, /*bmAttributes: bus powered and Support Remote Wake-up */
        0x32, /*MaxPower 100 mA: this current is used for detecting Vbus*/

        /************** Descriptor of MSC interface ****************/
        /* 09 */
        0x09,                          /*bLength: Interface Descriptor size*/
        USB_INTERFACE_DESCRIPTOR_TYPE, /*bDescriptorType: Interface descriptor type*/
        MSC_INTERFACE,                 /*bInterfaceNumber: Number of Interface*/
        0x00,                          /*bAlternateSetting: Alternate setting*/
        0x02,                          /*bNumEndpoints*/
        0x08,                          /* bInterfaceClass: MSC Class */
        0x06,                          /* bInterfaceSubClass : SCSI transparent*/
        0x50,                          /* nInterfaceProtocol */
        0x05,                          /*iInterface: Index of string descriptor*/
        /********************  Mass Storage Endpoints ********************/
        0x07,      /*Endpoint descriptor length = 7*/
        0x05,      /*Endpoint descriptor type */
        MSC_IN_EP, /*Endpoint address (IN, address 1) */
        0x02,      /*Bulk endpoint type */
        LOBYTE(MSC_MAX_PACKET),
        HIBYTE(MSC_MAX_PACKET),
        0x00, /*Polling interval in milliseconds */

        0x07,       /*Endpoint descriptor length = 7 */
        0x05,       /*Endpoint descriptor type */
        MSC_OUT_EP, /*Endpoint address (OUT, address 1) */
        0x02,       /*Bulk endpoint type */
        LOBYTE(MSC_MAX_PACKET),
        HIBYTE(MSC_MAX_PACKET),
        0x00, /*Polling interval in milliseconds*/

        /******** /IAD should be positioned just before the CDC interfaces ******
                    IAD to associate the two CDC interfaces */
        0x08, /* bLength */
        0x0B, /* bDescriptorType */
        0x01, /* bFirstInterface */
        0x02, /* bInterfaceCount */
        0x02, /* bFunctionClass */
        0x02, /* bFunctionSubClass */
        0x01, /* bFunctionProtocol */
        0x00, /* iFunction (Index of string descriptor describing this function) */

        /*Interface Descriptor */
        0x09,                          /* bLength: Interface Descriptor size */
        USB_INTERFACE_DESCRIPTOR_TYPE, /* bDescriptorType: Interface */
        /* Interface descriptor type */
        CDC_COM_INTERFACE, /* bInterfaceNumber: Number of Interface */
        0x00,              /* bAlternateSetting: Alternate setting */
        0x01,              /* bNumEndpoints: One endpoints used */
        0x02,              /* bInterfaceClass: Communication Interface Class */
        0x02,              /* bInterfaceSubClass: Abstract Control Model */
        0x01,              /* bInterfaceProtocol: Common AT commands */
        0x01,              /* iInterface: */

        /*Header Functional Descriptor*/
        0x05, /* bLength: Endpoint Descriptor size */
        0x24, /* bDescriptorType: CS_INTERFACE */
        0x00, /* bDescriptorSubtype: Header Func Desc */
        0x10, /* bcdCDC: spec release number */
        0x01,

        /*Call Management Functional Descriptor*/
        0x05, /* bFunctionLength */
        0x24, /* bDescriptorType: CS_INTERFACE */
        0x01, /* bDescriptorSubtype: Call Management Func Desc */
        0x00, /* bmCapabilities: D0+D1 */
        0x02, /* bDataInterface: 2 */

        /*ACM Functional Descriptor*/
        0x04, /* bFunctionLength */
        0x24, /* bDescriptorType: CS_INTERFACE */
        0x02, /* bDescriptorSubtype: Abstract Control Management desc */
        0x02, /* bmCapabilities */

        /*Union Functional Descriptor*/
        0x05, /* bFunctionLength */
        0x24, /* bDescriptorType: CS_INTERFACE */
        0x06, /* bDescriptorSubtype: Union func desc */
        0x01, /* bMasterInterface: Communication class interface */
        0x02, /* bSlaveInterface0: Data Class Interface */

        /*Endpoint 2 Descriptor*/
        0x07,                         /* bLength: Endpoint Descriptor size */
        USB_ENDPOINT_DESCRIPTOR_TYPE, /* bDescriptorType: Endpoint */
        CDC_CMD_EP,                   /* bEndpointAddress */
        0x03,                         /* bmAttributes: Interrupt */
        LOBYTE(CDC_CMD_PACKET_SIZE),  /* wMaxPacketSize: */
        HIBYTE(CDC_CMD_PACKET_SIZE),
        0xFF, /* bInterval: */

        /*---------------------------------------------------------------------------*/

        /*Data class interface descriptor*/
        0x09,                          /* bLength: Endpoint Descriptor size */
        USB_INTERFACE_DESCRIPTOR_TYPE, /* bDescriptorType: */
        0x02,                          /* bInterfaceNumber: Number of Interface */
        0x00,                          /* bAlternateSetting: Alternate setting */
        0x02,                          /* bNumEndpoints: Two endpoints used */
        0x0A,                          /* bInterfaceClass: CDC */
        0x00,                          /* bInterfaceSubClass: */
        0x00,                          /* bInterfaceProtocol: */
        0x00,                          /* iInterface: */

        /*Endpoint OUT Descriptor*/
        0x07,                             /* bLength: Endpoint Descriptor size */
        USB_ENDPOINT_DESCRIPTOR_TYPE,     /* bDescriptorType: Endpoint */
        CDC_OUT_EP,                       /* bEndpointAddress */
        0x02,                             /* bmAttributes: Bulk */
        LOBYTE(CDC_DATA_MAX_PACKET_SIZE), /* wMaxPacketSize: */
        HIBYTE(CDC_DATA_MAX_PACKET_SIZE),
        0x00, /* bInterval: ignore for Bulk transfer */

        /*Endpoint IN Descriptor*/
        0x07,                             /* bLength: Endpoint Descriptor size */
        USB_ENDPOINT_DESCRIPTOR_TYPE,     /* bDescriptorType: Endpoint */
        CDC_IN_EP,                        /* bEndpointAddress */
        0x02,                             /* bmAttributes: Bulk */
        LOBYTE(CDC_DATA_MAX_PACKET_SIZE), /* wMaxPacketSize: */
        HIBYTE(CDC_DATA_MAX_PACKET_SIZE),
        0x00, /* bInterval */
};

/**
 * @brief  USBD_MSC_CDC_Init
 *         Initialize the MSC & CDC interfaces
 * @param  pdev: device instance
 * @param  cfgidx: Configuration index
 * @retval status
 */
static uint8_t USBD_MSC_CDC_Init(void *pdev, uint8_t cfgidx)
{
    /* MSC initialization */
    USBD_MSC_Init(pdev, cfgidx);

    /* CDC initialization */
    USBD_CDC_Init(pdev, cfgidx);

    return USBD_OK;
}

/**
 * @brief  USBD_MSC_CDC_Init
 *         DeInitialize the MSC/CDC interfaces
 * @param  pdev: device instance
 * @param  cfgidx: Configuration index
 * @retval status
 */
static uint8_t USBD_MSC_CDC_DeInit(void *pdev, uint8_t cfgidx)
{
    /* MSC De-initialization */
    USBD_MSC_DeInit(pdev, cfgidx);

    /* CDC De-initialization */
    USBD_CDC_DeInit(pdev, cfgidx);

    return USBD_OK;
}

/**
 * @brief  USBD_MSC_CDC_Setup
 *         Handle the MSC/CDC specific requests
 * @param  pdev: instance
 * @param  req: usb requests
 * @retval status
 */
static uint8_t USBD_MSC_CDC_Setup(void *pdev, USB_SETUP_REQ *req)
{
    switch (req->bmRequest & USB_REQ_RECIPIENT_MASK)
    {
    case USB_REQ_RECIPIENT_INTERFACE:
        if (req->wIndex == MSC_INTERFACE)
        {
            return (USBD_MSC_Setup(pdev, req));
        }
        else
        {
            return (USBD_CDC_Setup(pdev, req));
        }

    case USB_REQ_RECIPIENT_ENDPOINT:
        if (req->wIndex == MSC_IN_EP)
        {
            return (USBD_MSC_Setup(pdev, req));
        }
        else
        {
            return (USBD_CDC_Setup(pdev, req));
        }

    default:
        break;
    }
    return USBD_OK;
}

/**
 * @brief  USBD_MSC_CDC_GetCfgDesc
 *         return configuration descriptor
 * @param  speed : current device speed
 * @param  length : pointer data length
 * @retval pointer to descriptor buffer
 */
static uint8_t *USBD_MSC_CDC_GetConfigDescriptor(uint8_t speed, uint16_t *length)
{
    *length = sizeof(USBD_MSC_CDC_CfgDesc);
    return USBD_MSC_CDC_CfgDesc;
}

/**
 * @brief  USBD_MSC_CDC_DataIn
 *         handle data IN Stage
 * @param  pdev: device instance
 * @param  epnum: endpoint index
 * @retval status
 */
static uint8_t USBD_MSC_CDC_DataIn(void *pdev, uint8_t epnum)
{
    /*DataIN can be for CDC or MSC */
    if (epnum == (CDC_IN_EP & 0x0F))
    {
        return (USBD_CDC_DataIn(pdev, epnum));
    }
    else
    {
        return (USBD_MSC_DataIn(pdev, epnum));
    }
}

static uint8_t USBD_MSC_CDC_DataOut(void *pdev, uint8_t epnum)
{
    /*DataOut can be for CDC */
    if (epnum == (CDC_OUT_EP & 0x0F))
    {
        return (USBD_CDC_DataOut(pdev, epnum));
    }
    else
    {
        return (USBD_MSC_DataOut(pdev, epnum));
    }
}

static uint8_t USBD_MSC_CDC_SOF(void *pdev)
{
    /*SOF processing needed for CDC */
    return (USBD_CDC_SOF(pdev));
}

static uint8_t USBD_MSC_CDC_EP0_RxReady(void *pdev)
{
    /*RxReady processing needed for CDC only*/
    return (USBD_CDC_EP0_RxReady(pdev));
}
