/**
*     Copyright (c) 2023, Nsing Technologies Plt. Ltd
*
*     All rights reserved.
*
*     This software is the exclusive property of Nsingechnologies Plt. Ltd (Hereinafter
* referred to as NSING). This software, and the product of NSING described herein
* (Hereinafter referred to as the Product) are owned by NSING under the laws and treaties
* of the People's Republic of China and other applicable jurisdictions worldwide.
*
*     NSING does not grant any license under its patents, copyrights, trademarks, or other
* intellectual property rights. Names and brands of third party may be mentioned or referred
* thereto (if any) for identification purposes only.
*
*     NSING reserves the right to make changes, corrections, enhancements, modifications, and
* improvements to this software at any time without notice. Please contact NSING and obtain
* the latest version of this software before placing orders.

*     Although NSING has attempted to provide accurate and reliable information, NSING assumes
* no responsibility for the accuracy and reliability of this software.
*
*     It is the responsibility of the user of this software to properly design, program, and test
* the functionality and safety of any application made of this information and any resulting product.
* In no event shall NSING be liable for any direct, indirect, incidental, special,exemplary, or
* consequential damages arising in any way out of the use of this software or the Product.
*
*     NSING Products are neither intended nor warranted for usage in systems or equipment, any
* malfunction or failure of which may cause loss of human life, bodily injury or severe property
* damage. Such applications are deemed, "Insecure Usage".
*
*     All Insecure Usage shall be made at user's risk. User shall indemnify NSING and hold NSING
* harmless from and against all claims, costs, damages, and other liabilities, arising from or related
* to any customer's Insecure Usage.

*     Any express or implied warranty with regard to this software or the Product, including,but not
* limited to, the warranties of merchantability, fitness for a particular purpose and non-infringement
* are disclaimed to the fullest extent permitted by law.

*     Unless otherwise explicitly permitted by NSING, anyone may not duplicate, modify, transcribe
* or otherwise distribute this software for any purposes, in whole or in part.
*
*     NSING products and technologies shall not be used for or incorporated into any products or systems
* whose manufacture, use, or sale is prohibited under any applicable domestic or foreign laws or regulations.
* User shall comply with any applicable export control laws and regulations promulgated and administered by
* the governments of any countries asserting jurisdiction over the parties or transactions.
**/

/**
 *\*\file usbd_def.h
 *\*\author Nsing
 *\*\version v1.0.0
 *\*\copyright Copyright (c) 2023, Nsingechnologies Plt. Ltd All rights reserved.
 **/
#ifndef __USBH_DEF_H__
#define __USBH_DEF_H__

#include "n32h47x_48x.h"

#ifndef USBH_NULL
#define USBH_NULL ((void *)0)
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define ValBit(VAR, POS) (VAR & (1 << POS))
#define SetBit(VAR, POS) (VAR |= (1 << POS))
#define ClrBit(VAR, POS) (VAR &= ((1 << POS) ^ 255))

#define LE16(addr) (((u16)(*((u8 *)(addr)))) + (((u16)(*(((u8 *)(addr)) + 1))) << 8))

#define USB_LEN_DESC_HDR 0x02
#define USB_LEN_DEV_DESC 0x12
#define USB_LEN_CFG_DESC 0x09
#define USB_LEN_IF_DESC 0x09
#define USB_LEN_EP_DESC 0x07
#define USB_LEN_SETUP_PKT 0x08

/* bmRequestType :D7 Data Phase Transfer Direction  */
#define USB_REQ_DIR_MASK 0x80
#define USB_H2D 0x00
#define USB_D2H 0x80

/* bmRequestType D6..5 Type */
#define USB_REQ_TYPE_STANDARD 0x00
#define USB_REQ_TYPE_CLASS 0x20
#define USB_REQ_TYPE_VENDOR 0x40
#define USB_REQ_TYPE_RESERVED 0x60

/* bmRequestType D4..0 Recipient */
#define USB_REQ_RECIPIENT_DEVICE 0x00
#define USB_REQ_RECIPIENT_INTERFACE 0x01
#define USB_REQ_RECIPIENT_ENDPOINT 0x02
#define USB_REQ_RECIPIENT_OTHER 0x03

/* Table 9-4. Standard Request Codes  */
/* bRequest , Value */
#define USB_REQ_GET_STATUS 0x00
#define USB_REQ_CLEAR_FEATURE 0x01
#define USB_REQ_SET_FEATURE 0x03
#define USB_REQ_SET_ADDRESS 0x05
#define USB_REQ_GET_DESCRIPTOR 0x06
#define USB_REQ_SET_DESCRIPTOR 0x07
#define USB_REQ_GET_CONFIGURATION 0x08
#define USB_REQ_SET_CONFIGURATION 0x09
#define USB_REQ_GET_INTERFACE 0x0A
#define USB_REQ_SET_INTERFACE 0x0B
#define USB_REQ_SYNCH_FRAME 0x0C

/* Table 9-5. Descriptor Types of USB Specifications */
#define USB_DESC_TYPE_DEVICE 1
#define USB_DESC_TYPE_CONFIGURATION 2
#define USB_DESC_TYPE_STRING 3
#define USB_DESC_TYPE_INTERFACE 4
#define USB_DESC_TYPE_ENDPOINT 5
#define USB_DESC_TYPE_DEVICE_QUALIFIER 6
#define USB_DESC_TYPE_OTHER_SPEED_CONFIGURATION 7
#define USB_DESC_TYPE_INTERFACE_POWER 8
#define USB_DESC_TYPE_HID 0x21
#define USB_DESC_TYPE_HID_REPORT 0x22

#define USB_DEVICE_DESC_SIZE 18
#define USB_CONFIGURATION_DESC_SIZE 9
#define USB_HID_DESC_SIZE 9
#define USB_INTERFACE_DESC_SIZE 9
#define USB_ENDPOINT_DESC_SIZE 7

/* Descriptor Type and Descriptor Index  */
/* Use the following values when calling the function USBH_GetDescriptor  */
#define USB_DESC_DEVICE ((USB_DESC_TYPE_DEVICE << 8) & 0xFF00)
#define USB_DESC_CONFIGURATION ((USB_DESC_TYPE_CONFIGURATION << 8) & 0xFF00)
#define USB_DESC_STRING ((USB_DESC_TYPE_STRING << 8) & 0xFF00)
#define USB_DESC_INTERFACE ((USB_DESC_TYPE_INTERFACE << 8) & 0xFF00)
#define USB_DESC_ENDPOINT ((USB_DESC_TYPE_INTERFACE << 8) & 0xFF00)
#define USB_DESC_DEVICE_QUALIFIER ((USB_DESC_TYPE_DEVICE_QUALIFIER << 8) & 0xFF00)
#define USB_DESC_OTHER_SPEED_CONFIGURATION ((USB_DESC_TYPE_OTHER_SPEED_CONFIGURATION << 8) & 0xFF00)
#define USB_DESC_INTERFACE_POWER ((USB_DESC_TYPE_INTERFACE_POWER << 8) & 0xFF00)
#define USB_DESC_HID_REPORT ((USB_DESC_TYPE_HID_REPORT << 8) & 0xFF00)
#define USB_DESC_HID ((USB_DESC_TYPE_HID << 8) & 0xFF00)

#define USB_EP_TYPE_CTRL 0x00
#define USB_EP_TYPE_ISOC 0x01
#define USB_EP_TYPE_BULK 0x02
#define USB_EP_TYPE_INTR 0x03

#define USB_EP_DIR_OUT 0x00
#define USB_EP_DIR_IN 0x80
#define USB_EP_DIR_MSK 0x80

/* supported classes */
#define USB_MSC_CLASS 0x08
#define USB_HID_CLASS 0x03

/* Interface Descriptor field values for HID Boot Protocol */
#define HID_BOOT_CODE 0x01
#define HID_KEYBRD_BOOT_CODE 0x01
#define HID_MOUSE_BOOT_CODE 0x02

/* As per USB specs 9.2.6.4 :Standard request with data request timeout: 5sec
   Standard request with no data stage timeout : 50ms */
#define DATA_STAGE_TIMEOUT 5000
#define NODATA_STAGE_TIMEOUT 50

/**
 * @}
 */

#define USBH_CONFIGURATION_DESCRIPTOR_SIZE (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE + (USBH_MAX_NUM_ENDPOINTS * USB_ENDPOINT_DESC_SIZE))

#define CONFIG_DESC_wTOTAL_LENGTH (ConfigurationDescriptorData.ConfigDescfield.ConfigurationDescriptor.wTotalLength)

/*  This Union is copied from usbhs_core.h  */
typedef union
{
  uint16_t w;
  struct BW
  {
    uint8_t msb;
    uint8_t lsb;
  } bw;
} uint16_t_uint8_t;

typedef union
{
  uint8_t d8[8];

  struct _SetupPkt_Struc
  {
    uint8_t bmRequestType;
    uint8_t bRequest;
    uint16_t_uint8_t wValue;
    uint16_t_uint8_t wIndex;
    uint16_t_uint8_t wLength;
  } b;
} USB_Setup_TypeDef;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
} USBH_DescHeader_t;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t bcdUSB; /* USB Specification Number which device complies too */
  uint8_t bDeviceClass;
  uint8_t bDeviceSubClass;
  uint8_t bDeviceProtocol;
  /* If equal to Zero, each interface specifies its own class
  code if equal to 0xFF, the class code is vendor specified.
  Otherwise field is valid Class Code.*/
  uint8_t bMaxPacketSize;
  uint16_t idVendor;          /* Vendor ID (Assigned by USB Org) */
  uint16_t idProduct;         /* Product ID (Assigned by Manufacturer) */
  uint16_t bcdDevice;         /* Device Release Number */
  uint8_t iManufacturer;      /* Index of Manufacturer String Descriptor */
  uint8_t iProduct;           /* Index of Product String Descriptor */
  uint8_t iSerialNumber;      /* Index of Serial Number String Descriptor */
  uint8_t bNumConfigurations; /* Number of Possible Configurations */
} USBH_DevDesc_TypeDef;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t wTotalLength;       /* Total Length of Data Returned */
  uint8_t bNumInterfaces;      /* Number of Interfaces */
  uint8_t bConfigurationValue; /* Value to use as an argument to select this configuration*/
  uint8_t iConfiguration;      /*Index of String Descriptor Describing this configuration */
  uint8_t bmAttributes;        /* D7 Bus Powered , D6 Self Powered, D5 Remote Wakeup , D4..0 Reserved (0)*/
  uint8_t bMaxPower;           /*Maximum Power Consumption */
} USBH_CfgDesc_TypeDef;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint16_t bcdHID;               /* indicates what endpoint this descriptor is describing */
  uint8_t bCountryCode;          /* specifies the transfer type. */
  uint8_t bNumDescriptors;       /* specifies the transfer type. */
  uint8_t bReportDescriptorType; /* Maximum Packet Size this endpoint is capable of sending or receiving */
  uint16_t wItemLength;          /* is used to specify the polling interval of certain transfers. */
} USBH_HIDDesc_TypeDef;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint8_t bInterfaceNumber;
  uint8_t bAlternateSetting;  /* Value used to select alternative setting */
  uint8_t bNumEndpoints;      /* Number of Endpoints used for this interface */
  uint8_t bInterfaceClass;    /* Class Code (Assigned by USB Org) */
  uint8_t bInterfaceSubClass; /* Subclass Code (Assigned by USB Org) */
  uint8_t bInterfaceProtocol; /* Protocol Code */
  uint8_t iInterface;         /* Index of String Descriptor Describing this interface */

} USBH_InterfaceDesc_TypeDef;

typedef struct
{
  uint8_t bLength;
  uint8_t bDescriptorType;
  uint8_t bEndpointAddress; /* indicates what endpoint this descriptor is describing */
  uint8_t bmAttributes;     /* specifies the transfer type. */
  uint16_t wMaxPacketSize;  /* Maximum Packet Size this endpoint is capable of sending or receiving */
  uint8_t bInterval;        /* is used to specify the polling interval of certain transfers. */
} USBH_EpDesc_TypeDef;
#endif
